using System;
using System.Windows.Forms;
using System.Drawing;
using System.Diagnostics;

namespace Example
{
	public abstract class Figure
	{
		protected float size = 30.0f;
		protected string id;
		protected int colorNr = 0;

		// drawing helpers
		protected readonly Color[] colors = { Color.Red, Color.Blue, Color.Green };
		protected Graphics grfx;
		protected float xPageMax, yPageMax;
		protected Pen pen;


		public float Size 
		{
			get { return size; }
			set { size = value; }
		}

		public string Id 
		{
			get { return id; }
			set { id = value; }
		}

		public int ColorNr 
		{
			get { return colorNr; }
			set { colorNr = value; }
		}


		public Figure()
		{
		}

		public virtual double Area()
		{
			return 0.0;
		}

		public virtual double Circumference()
		{
			return 0.0;
		}

		public virtual void Draw (Control control)
		{ 
			grfx = control.CreateGraphics();
			grfx.PageUnit = GraphicsUnit.Pixel;

			float yDevMax = grfx.VisibleClipBounds.Height;  // in pixel
			float pageScale;
			grfx.PageUnit = GraphicsUnit.Millimeter;
			// the hight of the panel corresponds to 100 mm ( = yPageMax )
			yPageMax = 100.0f;
			// transformation from page to device coordinates:
			// yDev = yPage x PageScale x DpiY / 25.4
			// yDevMax = yPageMax x PageScale x DpiY / 25.4
			// ==>
			// PageScale = yDevMax / yPageMax / DpiY * 25.4
			pageScale = yDevMax / yPageMax / grfx.DpiY * 25.4f; // for size = 10 to 100
			// for sizes > 100 or < 10 we calculate a zoom factor
			// excercise: Try to understand the formulas in the next two lines
			double exp = Math.Floor(Math.Log10(size / yPageMax)) + 1;
			float zoom = (float) Math.Pow (10.0, exp);
			grfx.PageScale = pageScale/zoom;
			xPageMax = grfx.VisibleClipBounds.Width;
			yPageMax = grfx.VisibleClipBounds.Height;
			//	Debug.WriteLine(String.Format("exp = {0} zoom = {1} xPageMax = {2}, yPagMax = {3}", 
			//		f, zoom, xPageMax, yPageMax));
			Font font = new Font(control.Font.FontFamily, 10.0f);

			string s = String.Format("A = {0:f2}\nu = {1:f2}", Area(), Circumference());
			grfx.DrawString(s, font, Brushes.Black, 0.7f*xPageMax, 0.1f*yPageMax);
			if (zoom >= 1.0f)
				s = String.Format("Zoom 1:{0:f0}", zoom);
			else
				s = String.Format("Zoom {0:f0}:1", 1.0f/zoom);

			grfx.DrawString(s, font, Brushes.Black, 0.7f*xPageMax, 0.9f*yPageMax);

			pen = new Pen(Color.Black , yPageMax/200.0f);
			
		}
	}

	///////////////////////////////////////////////////////////////////////////

	public class Square : Figure
	{
		public Square()
		{
		}

		public override double Area()
		{
			return size*size;
		}

		public override double Circumference()
		{
			return 4*size;
		}

		public override void Draw(Control control)
		{
			base.Draw(control);
			pen.Color= colors[colorNr];
			grfx.DrawRectangle(pen, yPageMax/2-size/2, yPageMax/2-size/2, size, size);
			// Clean up the Graphics object.
			grfx.Dispose();

		}
	}

	///////////////////////////////////////////////////////////////////////////

	public class Circle : Figure
	{
		public Circle()
		{
		}

		public override double Area()
		{
			return size*size*Math.PI;
		}

		public override double Circumference()
		{
			return 2*size*Math.PI;
		}


		public override void Draw(Control control)
		{
			base.Draw(control);
			pen.Color= colors[colorNr];
			grfx.DrawEllipse(pen, yPageMax/2 - size/2, yPageMax/2 - size/2, size, size);
			grfx.Dispose();
		}

	}

	///////////////////////////////////////////////////////////////////////////

	public class Triangle : Figure
	{
		public Triangle()
		{
		}

		public override double Area()
		{
			return size*size*Math.Sqrt(3.0)/4.0;
		}

		public override double Circumference()
		{
			return 3*size;
		}

		public override void Draw(Control control)
		{
			base.Draw(control);
			pen.Color= colors[colorNr];
			float h = size * (float) (Math.Sqrt(3.0)/2.0);
			PointF[] pts = {
			  new PointF (yPageMax/2 - size/2, yPageMax/2 + h/3),
			  new PointF (yPageMax/2 + size/2, yPageMax/2 + h/3),
			  new PointF (yPageMax/2,          yPageMax/2 - 2*h/3),
			  new PointF (yPageMax/2 - size/2, yPageMax/2 + h/3),
			};
			grfx.DrawLines(pen, pts);
			grfx.Dispose();
		}

	}

}
